# -*- coding: utf-8 -*-

from imio.smartweb.core.testing import IMIO_SMARTWEB_CORE_INTEGRATION_TESTING
from imio.smartweb.core.testing import ImioSmartwebTestCase
from imio.smartweb.core.tests.utils import make_named_image
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.namedfile.file import NamedBlobImage
from zope.component import queryMultiAdapter


class TestSectionGallery(ImioSmartwebTestCase):
    layer = IMIO_SMARTWEB_CORE_INTEGRATION_TESTING

    def setUp(self):
        self.request = self.layer["request"]
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.page = api.content.create(
            container=self.portal,
            type="imio.smartweb.Page",
            id="page",
        )

    def test_alt_label(self):
        gallery_section = api.content.create(
            container=self.page,
            type="imio.smartweb.SectionGallery",
            title="Gallery section",
        )
        blob_image = NamedBlobImage(**make_named_image("plone.png"))

        # image1 has no defined title (by default img blob filename) and no description
        image1 = api.content.create(
            container=gallery_section,
            type="Image",
            title="plone.png",
        )
        image1.image = blob_image

        # image2 has a defined title and no description
        image2 = api.content.create(
            container=gallery_section,
            type="Image",
            title="Kamoulox",
        )
        image2.image = blob_image

        # image3 has a defined title and description (greater than title)
        image3 = api.content.create(
            container=gallery_section,
            type="Image",
            title="Hello Plone !",
        )
        image3.image = blob_image
        image3.description = "Hello Plone, what's up ?!"

        # image4 has a defined title and a mistaked blank description (smaller than title)
        image4 = api.content.create(
            container=gallery_section,
            type="Image",
            title="Hello Plone !",
        )
        # This is a mistaked blank description
        image4.image = blob_image
        image4.description = "  "

        view = queryMultiAdapter((gallery_section, self.request), name="view")
        self.assertEqual(view.alt_label(image1), "")
        self.assertEqual(view.alt_label(image2), "Kamoulox")
        self.assertEqual(view.alt_label(image3), "Hello Plone, what's up ?!")
        self.assertEqual(view.alt_label(image4), "Hello Plone !")



# -*- coding: utf-8 -*-

from imio.smartweb.common.utils import get_vocabulary
from imio.smartweb.core.contents.rest.base import BaseEndpoint
from imio.smartweb.core.contents.rest.base import BaseService
from imio.smartweb.core.utils import get_basic_auth_json
from imio.smartweb.core.utils import get_ts_api_url
from plone import api
from plone.i18n.normalizer import idnormalizer
from plone.restapi.interfaces import IExpandableElement
from zope.component import adapter
from zope.i18n import translate
from zope.interface import implementer
from zope.interface import Interface

import base64
import requests


PROJECT_WORKFLOW_STATUS_TO_KEEP = [
    "Publié sans vote",
    "Publié",
    "Vote",
    "Clôture",
    "Retenu",
    "Rejeté",
]


def get_ideabox_basic_auth_header() -> str:
    user = api.portal.get_registry_record("smartweb.iaideabox_api_username")
    pwd = api.portal.get_registry_record("smartweb.iaideabox_api_password")
    usrPass = f"{user}:{pwd}".encode("utf-8")
    b64Val = base64.b64encode(usrPass)
    return f"Basic {b64Val.decode('utf-8')}"


class BaseTsEndpoint(BaseEndpoint):
    def __init__(self, *args, **kwargs):
        super(BaseTsEndpoint, self).__init__(*args, **kwargs)
        self.user = api.portal.get_registry_record("smartweb.iaideabox_api_username")
        self.pwd = api.portal.get_registry_record("smartweb.iaideabox_api_password")

    def _get_json(self, url):
        return get_basic_auth_json(url, self.user, self.pwd)

    @property
    def query_url(self):
        raise NotImplementedError("query_url must be defined in subclasses")


@implementer(IExpandableElement)
@adapter(Interface, Interface)
class CampaignEndpoint(BaseTsEndpoint):

    def __call__(self):
        require_keys = [
            "uuid",
            "id",
            "display_name",
            "text",
            "url",
            "api_url",
            "fields",
            "workflow",
        ]
        json = self._get_json(self.query_url)
        json_res = {}
        if not json:
            return {}
        elif "data" in json:
            # list of projects
            json["data"] = self.add_b64_image_to_data(json.get("data"))
            projects = [
                {k: v for k, v in d.items() if k in require_keys}
                for d in json.get("data")
            ]
            json_res["items"] = projects
            json_res["items_total"] = json.get("count")

        else:
            # single project
            json.pop("evolution")
            json.pop("roles")
            json.pop("submission")
            json.pop("user")
            json_res = json
        # Return a json which represents a project or which contains a list of projects
        return json_res

    def add_b64_image_to_data(self, data):
        for d in data:
            # get image
            image_url = d.get("fields").get("images_raw")[0].get("image").get("url")
            content = self.get_image(image_url).content
            b64_content = base64.b64encode(content).decode("utf-8")
            d["fields"]["images_raw"][0]["image"]["content"] = b64_content
        return data

    def get_image(self, image_url):
        if not image_url:
            return
        headers = {"Accept": "image/*"}
        headers["Authorization"] = get_ideabox_basic_auth_header()
        response = requests.get(image_url, headers=headers)
        return response

    @property
    def query_url(self):
        wcs_api = get_ts_api_url("wcs")
        if "id" in self.request.form:
            # we want a specific project
            project_id = self.request.form.get("id")
            url = f"{wcs_api}/cards/imio-ideabox-projet/{project_id}?full=on"
        else:
            # we want (filtered) list of projects for a specific campaign with eventually extra pamams (zone, topic, ...)
            extra_params = [f"{k}={v}" for k, v in self.request.form.items()]
            extra_params = "&".join(extra_params)
            campaign_id = self.context.linked_campaign
            filter_statut = "|".join(PROJECT_WORKFLOW_STATUS_TO_KEEP)
            url = f"{wcs_api}/cards/imio-ideabox-projet/list?filter-campagne={campaign_id}&full=on&filter-statut={filter_statut}&filter-statut-operator=in&{extra_params}"
        return url


@implementer(IExpandableElement)
@adapter(Interface, Interface)
class ZonesEndpoint(BaseTsEndpoint):

    def __call__(self):
        json_res = {"items": [], "items_total": 0}
        json = self._get_json(self.query_url)
        if not json:
            return json_res
        json_res = {"items": json["data"], "items_total": json.get("count")}
        return json_res

    @property
    def query_url(self):
        wcs_api = get_ts_api_url("wcs")
        campaign_id = self.context.linked_campaign
        return f"{wcs_api}/cards/imio-ideabox-zone/list?filter-campagne={campaign_id}"


@implementer(IExpandableElement)
@adapter(Interface, Interface)
class TsTopicsEndpoint(BaseTsEndpoint):

    def __call__(self):
        json_res = {"items": [], "items_total": 0}
        json = self._get_json(self.query_url)
        if not json:
            return json_res
        json_res = {"items": json["data"], "items_total": json.get("count")}
        return json_res

    @property
    def query_url(self):
        wcs_api = get_ts_api_url("wcs")
        campaign_id = self.context.linked_campaign
        return f"{wcs_api}/cards/imio-ideabox-theme/list?filter-campagne={campaign_id}"


@implementer(IExpandableElement)
@adapter(Interface, Interface)
class TopicsEndpoint(BaseEndpoint):

    def __call__(self):
        # get all topics from smartweb vocabulary (populate select in React campaign view)
        topics_vocabulary = get_vocabulary("imio.smartweb.vocabulary.Topics")
        current_lang = api.portal.get_current_language()[:2]
        json_res = {
            "items": [
                {
                    "value": term.value,
                    "title": translate(term.title, target_language=current_lang),
                }
                for term in topics_vocabulary
            ],
            "items_total": len(topics_vocabulary),
        }
        return json_res


@implementer(IExpandableElement)
@adapter(Interface, Interface)
class AllTopicsEndpoint(TopicsEndpoint):

    def __call__(self):
        topics = super(AllTopicsEndpoint, self).__call__()
        ts_endpoint = TsTopicsEndpoint(self.context, self.request)
        ts_topics = ts_endpoint()
        ts_topics = [
            {"title": i.get("text"), "value": i.get("id")} for i in ts_topics["items"]
        ]
        topics["items"].extend(ts_topics)
        topics["items"].sort(key=lambda x: idnormalizer.normalize(x["title"]))
        topics["items_total"] += len(ts_topics)
        return topics


class CampaignEndpointGet(BaseService):
    def reply(self):
        return CampaignEndpoint(self.context, self.request)()


class AuthCampaignEndpointGet(BaseService):
    def reply(self):
        return get_ideabox_basic_auth_header()


class ZonesEndpointGet(BaseService):
    def reply(self):
        return ZonesEndpoint(self.context, self.request)()


class TsTopicsEndpointGet(BaseService):
    def reply(self):
        return ZonesEndpoint(self.context, self.request)()


class TopicsEndpointGet(BaseService):
    def reply(self):
        return TopicsEndpoint(self.context, self.request)()


class AllTopicsEndpointGet(BaseService):
    def reply(self):
        return AllTopicsEndpoint(self.context, self.request)()



# -*- coding: utf-8 -*-

from imio.smartweb.core.contents import (
    CampaignView,
    DirectoryView,
    EventsView,
    NewsView,
)
from plone.app.contentmenu.menu import ActionsSubMenuItem as BaseActionsSubMenuItem
from plone.app.contentmenu.menu import DisplaySubMenuItem as BaseDisplaySubMenuItem
from plone.app.layout.viewlets.common import (
    ContentViewsViewlet as BaseContentViewsViewlet,
)
from plone.memoize.instance import memoize


class ContentViewsViewlet(BaseContentViewsViewlet):
    """Add preview action to primary actions to ensure ordering"""

    primary = ["folderContents", "edit", "view", "preview"]


class ExcludeSubMenuForCertainTypesMixin:
    """Mixin pour exclure certaines vues des sous-menus."""

    excluded_views = (CampaignView, DirectoryView, EventsView, NewsView)

    @memoize
    def available(self):
        return super().available() and not isinstance(self.context, self.excluded_views)


class ActionsSubMenuItem(ExcludeSubMenuForCertainTypesMixin, BaseActionsSubMenuItem):
    """Don't show actions sub-menu for authentic sources views."""


class DisplaySubMenuItem(ExcludeSubMenuForCertainTypesMixin, BaseDisplaySubMenuItem):
    """Don't show display sub-menu for authentic sources views."""



from Acquisition import aq_inner
from BTrees.OOBTree import OOBTree
from imio.smartweb.core.contents.rest.directory.endpoint import DirectoryEndpointGet
from imio.smartweb.core.contents.rest.events.endpoint import EventsEndpointGet
from imio.smartweb.core.contents.rest.news.endpoint import NewsEndpointGet
from imio.smartweb.core.interfaces import IImioSmartwebCoreLayer
from plone.app.layout.navigation.navtree import buildFolderTree
from plone.app.layout.sitemap.sitemap import SiteMapView
from plone.base.interfaces import IPloneSiteRoot
from plone.memoize import ram
from Products.CMFPlone.browser.navigation import CatalogSiteMap as BaseCatalogSiteMap
from Products.CMFPlone.browser.navtree import (
    SitemapNavtreeStrategy as BaseSitemapNavtreeStrategy,
)
from Products.CMFPlone.browser.navtree import (
    SitemapQueryBuilder as BaseSitemapQueryBuilder,
)
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.utils import normalizeString
from zope.component import getMultiAdapter
from zope.interface import implementer
from zope.interface import Interface

import logging
import Missing
import time

logger = logging.getLogger("imio.smartweb.core")


FRIENDLY_TYPES = [
    "Collection",
    "Image",
    "Link",
    "imio.smartweb.Folder",
    "imio.smartweb.Page",
    "imio.smartweb.PortalPage",
    "imio.smartweb.Procedure",
    "imio.smartweb.CirkwiView",
    "imio.smartweb.DirectoryView",
    "imio.smartweb.EventsView",
    "imio.smartweb.NewsView",
]


def cache_key(method, obj, request):
    """We cache data from authentic sources for the sitemap (.xml.gz) for 2 hours."""
    return f"sitemap_{obj.UID()}_{time.time() // 7200}"


@ram.cache(cache_key)
def get_endpoint_data(obj, request):
    endpoint_mapping = {
        "imio.smartweb.DirectoryView": DirectoryEndpointGet,
        "imio.smartweb.EventsView": EventsEndpointGet,
        "imio.smartweb.NewsView": NewsEndpointGet,
    }
    endpoint_class = endpoint_mapping.get(obj.portal_type)
    if not endpoint_class:
        return {}

    endpoint = endpoint_class()
    batch_size = 1000 if obj.portal_type == "imio.smartweb.DirectoryView" else 365
    return (
        endpoint.reply_for_given_object(
            obj, request, fullobjects=0, batch_size=batch_size
        )
        or {}
    )


def format_sitemap_items(items, base_url):
    """Format items for sitemap(.xml.gz)"""
    formatted_items = []
    for item in items:
        item_id = normalizeString(item.get("title"))
        item_uid = item.get("id")
        lastmod = item.get("modified") or "1970-01-01T00:00:00Z"
        formatted_items.append(
            {
                "loc": f"{base_url}/{item_id}?u={item_uid}",
                "lastmod": lastmod,
                "Title": item.get("title"),
                "Description": item.get("description", ""),
                "getURL": f"{base_url}/{item_id}?u={item_uid}",
                "getRemoteUrl": Missing.Value,
                "currentItem": False,
                "currentParent": False,
                "normalized_review_state": "published",
                "normalized_portal_type": "imio-smartweb-authsources-item",
            }
        )
    return formatted_items


class CustomSiteMapView(SiteMapView):
    """Custom sitemap view. (get items for sitemap.xml.gz)"""

    def objects(self):
        """Returns the data to create the sitemap."""
        friendlytypes = FRIENDLY_TYPES

        catalog = getToolByName(self.context, "portal_catalog")
        query = {
            "portal_type": getToolByName(
                self.context, "plone_utils"
            ).getUserFriendlyTypes(friendlytypes)
        }

        is_plone_site_root = IPloneSiteRoot.providedBy(self.context)
        if not is_plone_site_root:
            query["path"] = "/".join(self.context.getPhysicalPath())

        query["is_default_page"] = True
        default_page_modified = OOBTree()
        for item in catalog.searchResults(query):
            key = item.getURL().rsplit("/", 1)[0]
            value = (item.modified.micros(), item.modified.ISO8601())
            default_page_modified[key] = value

        if is_plone_site_root:
            loc = self.context.absolute_url()
            date = self.context.modified()
            modified = max(
                (date.micros(), date.ISO8601()), default_page_modified.get(loc, (0, ""))
            )
            yield {"loc": loc, "lastmod": modified[1]}

        query["is_default_page"] = False
        for item in catalog.searchResults(query):
            loc = item.getURL()
            date = item.modified
            modified = max(
                (date.micros(), date.ISO8601()), default_page_modified.get(loc, (0, ""))
            )
            yield {"loc": loc, "lastmod": modified[1]}

        brains = catalog(
            portal_type=[
                "imio.smartweb.EventsView",
                "imio.smartweb.NewsView",
                "imio.smartweb.DirectoryView",
            ]
        )
        for brain in brains:
            obj = brain.getObject()
            data = get_endpoint_data(obj, obj.REQUEST)
            yield from format_sitemap_items(data.get("items", {}), obj.absolute_url())


@implementer(IImioSmartwebCoreLayer)
class CatalogSiteMap(BaseCatalogSiteMap):
    def siteMap(self):
        context = aq_inner(self.context)

        queryBuilder = SitemapQueryBuilder(context)
        query = queryBuilder()
        strategy = getMultiAdapter((context, self), ISmartwebNavtreeStrategy)
        base_folder_tree = buildFolderTree(
            context, obj=context, query=query, strategy=strategy
        )

        for child in base_folder_tree.get("children"):
            obj = child.get("item").getObject()
            data = get_endpoint_data(obj, obj.REQUEST)
            if not data:
                continue
            child["children"] = format_sitemap_items(
                data.get("items", []), obj.absolute_url()
            )

        return base_folder_tree


class SitemapQueryBuilder(BaseSitemapQueryBuilder):
    """Builds the request for the sitemap with custom content types."""

    def __init__(self, context):
        self.context = context

    def __call__(self):
        query = {}
        custom_query = getattr(self.context, "getCustomNavQuery", None)
        if custom_query and callable(custom_query):
            query = custom_query()
        query["portal_type"] = FRIENDLY_TYPES
        query["review_state"] = "published"
        return query


class ISmartwebNavtreeStrategy(Interface):
    """Marker interface for the smartweb navtree strategy."""


@implementer(ISmartwebNavtreeStrategy)
class SitemapNavtreeStrategy(BaseSitemapNavtreeStrategy):

    def nodeFilter(self, node):
        # Return all nodes in sitemap even if they are exclude from nav
        return True



# -*- coding: utf-8 -*-

# from imio.smartweb.core.contents import IPortalPage
from plone.app.layout.viewlets.social import SocialTagsViewlet as BaseSocialTagsViewlet


class SocialTagsViewlet(BaseSocialTagsViewlet):
    social_image_scale = "paysage_vignette"

    @property
    def tags(self):
        tags = super(SocialTagsViewlet, self).tags
        # if IPortalPage.providedBy(self.context):
        if getattr(self.context, "image", None) is None:
            return tags
        tags = [item for item in tags if "og:image" not in item.get("property", "")]
        scales = self.context.restrictedTraverse("@@images")
        scale = scales.scale("image", scale=self.social_image_scale)
        scale_url = scale.absolute_url()
        image = self.context.image
        tags.extend(
            [
                dict(property="og:image", content=scale_url),
                dict(property="og:image:width", content=scale.width),
                dict(property="og:image:height", content=scale.height),
                dict(itemprop="image", content=scale_url),
                dict(property="og:image:type", content=image.contentType),
            ]
        )
        return tags



# -*- coding: utf-8 -*-

from collective.instancebehavior.interfaces import IInstanceBehaviorAssignableContent
from imio.smartweb.core.contents import RestView
from imio.smartweb.core.utils import get_basic_auth_json
from imio.smartweb.core.utils import get_ts_api_url
from imio.smartweb.core.utils import get_value_from_registry
from imio.smartweb.locales import SmartwebMessageFactory as _
from plone import api
from plone.app.content.namechooser import NormalizingNameChooser
from plone.i18n.normalizer import idnormalizer
from plone.supermodel import model
from zope import schema
from zope.component import getUtility
from zope.container.interfaces import INameChooser
from zope.interface import implementer

import logging

logger = logging.getLogger("imio.smartweb.core")


class ICampaignView(model.Schema):
    """ """

    linked_campaign = schema.Choice(
        vocabulary="imio.smartweb.vocabulary.PublikCampaigns",
        title=_("E-Guichet campaign"),
        required=False,
        default=None,
    )

    propose_project_url = schema.TextLine(
        title=_("Propose project URL"),
        description=_("URL to propose a project"),
        required=False,
    )

    nb_results = schema.Int(
        title=_("Number of items to display"), default=20, required=True
    )

    display_map = schema.Bool(
        title=_("Display map"),
        description=_("If selected, map will be displayed"),
        required=False,
        default=True,
    )

    display_red_thumbs = schema.Bool(
        title=_("Display red thumbs"),
        description=_("If selected, red thumbs will be displayed"),
        required=False,
        default=False,
    )


@implementer(ICampaignView, IInstanceBehaviorAssignableContent)
class CampaignView(RestView):
    """Campaign class"""


@implementer(INameChooser)
class CampaignNameChooser(NormalizingNameChooser):
    def chooseName(self, name, obj):
        """Génère un ID basé sur le titre récupéré de l'API."""
        if ICampaignView.providedBy(obj):
            if not obj.title:
                # Récupération du titre via l'API
                wcs_api = get_ts_api_url("wcs")
                ts_campaign_endpoint = "imio-ideabox-campagne"
                url = f"{wcs_api}/cards/{ts_campaign_endpoint}/{obj.linked_campaign}"
                user = get_value_from_registry("smartweb.iaideabox_api_username")
                pwd = get_value_from_registry("smartweb.iaideabox_api_password")
                json_campaign = get_basic_auth_json(url, user, pwd)

                if json_campaign and "fields" in json_campaign:
                    obj.title = json_campaign["fields"].get(
                        "titre", "campagne-sans-nom"
                    )
                else:
                    obj.title = "campaign-without-name"

            # Normaliser l'ID en utilisant le titre
            normalized_id = idnormalizer.normalize(obj.title)

            # Vérifier que l'ID généré est bien unique dans le conteneur
            return super().chooseName(normalized_id, obj)

        return super().chooseName(name, obj)



# -*- coding: utf-8 -*-

from collective.z3cform.datagridfield.datagridfield import DataGridFieldFactory
from collective.z3cform.datagridfield.registry import DictRow
from imio.smartweb.locales import SmartwebMessageFactory as _
from plone import api
from plone.app.registry.browser.controlpanel import ControlPanelFormWrapper
from plone.app.registry.browser.controlpanel import RegistryEditForm
from plone.autoform import directives
from plone.z3cform import layout
from zope import schema
from z3c.form import field
from z3c.form.browser.password import PasswordFieldWidget
from zope.interface import Interface


class ISendinblueTextRowSchema(Interface):
    language = schema.TextLine(
        title=_("Language (en, fr,...)"),
        description=_("Enter the language code. Ex.: en"),
    )

    text = schema.TextLine(title=_("Text"), description=_("Your button title"))


class ISmartwebControlPanel(Interface):
    # https://COMMUNE-formulaires.guichet-citoyen.be/api
    url_ts = schema.TextLine(
        title=_("Url to e-guichet"),
        description=_("Example : https://COMMUNE.guichet-citoyen.be"),
        required=False,
    )

    secret_key_api = schema.Password(
        title=_("Secret key"),
        description=_("Secret key to use API"),
        required=False,
    )

    iaideabox_api_username = schema.TextLine(
        title=_(
            "Username to consume e-guichet ideabox API (get Campaign, projects,...)"
        ),
        default="ideabox",
        required=False,
    )

    iaideabox_api_password = schema.Password(
        title=_(
            "Password to consume e-guichet ideabox API (get Campaign, projects,...)"
        ),
        required=False,
    )

    propose_directory_url = schema.URI(
        title=_("Url to propose a new citizen contact"),
        required=False,
    )

    propose_events_url = schema.URI(
        title=_("Url to propose a new citizen event"),
        required=False,
    )

    propose_news_url = schema.URI(
        title=_("Url to propose a new citizen news"),
        required=False,
    )

    directory_entity_uid = schema.Choice(
        title=_("Directory entity"),
        source="imio.smartweb.vocabulary.RemoteDirectoryEntities",
        required=False,
    )

    events_entity_uid = schema.Choice(
        title=_("Events entity"),
        source="imio.smartweb.vocabulary.RemoteEventsEntities",
        required=False,
    )

    news_entity_uid = schema.Choice(
        title=_("News entity"),
        source="imio.smartweb.vocabulary.RemoteNewsEntities",
        required=False,
    )

    directory_solr_core = schema.TextLine(
        title=_("Directory SolR Core ID"),
        default="directory",
        required=False,
    )

    events_solr_core = schema.TextLine(
        title=_("Events SolR Core ID"),
        default="events",
        required=False,
    )

    news_solr_core = schema.TextLine(
        title=_("News SolR Core ID"),
        default="news",
        required=False,
    )

    default_directory_view = schema.Choice(
        title=_("Default directory view"),
        description=_("This information is used for search results redirection"),
        source="imio.smartweb.vocabulary.DirectoryViews",
        required=False,
    )

    default_events_view = schema.Choice(
        title=_("Default events view"),
        description=_("This information is used for search results redirection"),
        source="imio.smartweb.vocabulary.EventsViews",
        required=False,
    )

    default_news_view = schema.Choice(
        title=_("Default news view"),
        description=_("This information is used for search results redirection"),
        source="imio.smartweb.vocabulary.NewsViews",
        required=False,
    )

    category_and_topics_vocabulary = schema.Dict(
        title=_("Category and topics vocabulary setting for TeleServices"),
        description=_(
            "Choose a term from the vocabulary and define the corresponding name"
        ),
        key_type=schema.Choice(
            title=_("Vocabulary term"),
            source="imio.smartweb.vocabulary.CategoryAndTopics",
        ),
        value_type=schema.TextLine(
            title=_("Name"),
        ),
        required=False,
    )

    sendinblue_button_position = schema.Choice(
        title=_("SendInBlue : Define button position"),
        source="imio.smartweb.vocabulary.SendInBlueButtonPosition",
        default="button_bottom",
        required=True,
    )

    sendinblue_button_text = schema.List(
        title=_("SendInBlue : Define button text"),
        description=_("Choose SendInblue submission button text"),
        value_type=DictRow(
            title="Value",
            schema=ISendinblueTextRowSchema,
        ),
        default=[],
        required=True,
    )
    directives.widget("sendinblue_button_text", DataGridFieldFactory, auto_append=False)

    plausible_url = schema.TextLine(
        title=_("Plausible URL"),
        description=_(
            "Example : plausible.imio.be (SMARTWEB_PLAUSIBLE_URL varenv has precedence over this.)"
        ),
        required=False,
    )

    plausible_site = schema.TextLine(
        title=_("Plausible Site"),
        description=_(
            "Example : namur.be (SMARTWEB_PLAUSIBLE_SITE varenv has precedence over this.)"
        ),
        required=False,
    )

    plausible_token = schema.TextLine(
        title=_("Plausible Token"),
        description=_(
            "Plausible authentification token (SMARTWEB_PLAUSIBLE_TOKEN varenv has precedence over this.)"
        ),
        required=False,
    )

    iadeliberations_url = schema.TextLine(
        title=_("I.A. Deliberations' url"),
        description=_(
            "For staging : https://conseil.staging.imio.be | for production : https://www.deliberations.be"
        ),
        default="https://www.deliberations.be",
        required=False,
    )

    iadeliberations_api_username = schema.TextLine(
        title=_("Username to consume I.A. Deliberations API"),
        default="wsviewersmartweb",
        required=False,
    )

    iadeliberation_api_password = schema.Password(
        title=_("Password to consume I.A. Deliberations API"),
        required=False,
    )

    iadeliberations_institution = schema.Choice(
        title=_("I.A. Deliberations : Institutions"),
        source="imio.smartweb.vocabulary.IADeliberationsInstitutions",
        required=False,
    )


class SmartwebControlPanelForm(RegistryEditForm):
    schema = ISmartwebControlPanel
    schema_prefix = "smartweb"
    label = _("Smartweb Settings")
    fields = field.Fields(ISmartwebControlPanel)
    fields["secret_key_api"].widgetFactory = PasswordFieldWidget
    fields["iaideabox_api_password"].widgetFactory = PasswordFieldWidget
    fields["iadeliberation_api_password"].widgetFactory = PasswordFieldWidget

    def applyChanges(self, data):
        # Get current registry value for (controlpanel) passwords
        iadeliberation_pwd = api.portal.get_registry_record(
            "smartweb.iadeliberation_api_password"
        )

        iaideabox_pwd = api.portal.get_registry_record(
            "smartweb.iaideabox_api_password"
        )

        secret_key_api = api.portal.get_registry_record("smartweb.secret_key_api")

        # if data is None when we apply changes for password fields we keep password from registry
        if not data.get("iadeliberation_api_password"):
            data["iadeliberation_api_password"] = iadeliberation_pwd

        if not data.get("iaideabox_api_password"):
            data["iaideabox_api_password"] = iaideabox_pwd

        if not data.get("secret_key_api"):
            data["secret_key_api"] = secret_key_api

        return super().applyChanges(data)

    def updateFields(self):
        """Override updateFields to hide fields based on ideabox profile installation."""
        super(SmartwebControlPanelForm, self).updateFields()
        # Check if the 'ideabox' profile is installed
        portal_setup = api.portal.get_tool(name="portal_setup")
        profile_version = portal_setup.getLastVersionForProfile(
            "imio.smartweb.core:ideabox"
        )

        if profile_version == "unknown":
            # Hide the field related to Idea Box if the profile is not installed
            self.fields = self.fields.omit("iaideabox_api_username")
            self.fields = self.fields.omit("iaideabox_api_password")

    def updateWidgets(self):
        super(SmartwebControlPanelForm, self).updateWidgets()
        password_fields = [
            "secret_key_api",
            "iaideabox_api_password",
            "iadeliberation_api_password",
        ]
        for widget_name in self.widgets:
            widget = self.widgets[widget_name]
            if widget_name in password_fields:
                field_value = api.portal.get_registry_record(f"smartweb.{widget_name}")
                if field_value:
                    widget.placeholder = _("Password already filled")
                    widget.klass = (
                        f"{widget.klass} password-already-filled"
                        if widget.klass
                        else "password-already-filled"
                    )


SmartwebControlPanelView = layout.wrap_form(
    SmartwebControlPanelForm, ControlPanelFormWrapper
)



# -*- coding: utf-8 -*-

from imio.smartweb.locales import SmartwebMessageFactory as _
from plone import api
from plone.app.layout.viewlets import common
from zope.component import getMultiAdapter
from zope.i18n import translate
from zope.viewlet.interfaces import IViewletManager


class HeaderActionsViewlet(common.ViewletBase):
    def update(self):
        context_state = getMultiAdapter(
            (self.context, self.request), name="plone_context_state"
        )
        language = api.portal.get_current_language(context=self.context)
        self.actions = context_state.actions("header_actions")
        for action in self.actions:
            if action["id"] == "account":
                action["link_target"] = "_blank"
                desc = translate(_(action["description"]), target_language=language)
                new_tab = translate(_("New tab"), target_language=language)
                action["description"] = f"{desc} ({new_tab})"


class HeaderActionsContainerViewlet(common.ViewletBase):
    """Viewlet containing the header actions viewlets manager"""


class IHeaderActionsViewletsManager(IViewletManager):
    """Viewlet manager containing header actions viewlets"""



# -*- coding: utf-8 -*-

from freezegun import freeze_time
from gzip import GzipFile
from io import BytesIO
from imio.smartweb.core.browser.sitemap import CatalogSiteMap
from imio.smartweb.core.browser.sitemap import get_endpoint_data
from imio.smartweb.core.browser.sitemap import cache_key
from imio.smartweb.core.testing import IMIO_SMARTWEB_CORE_FUNCTIONAL_TESTING
from imio.smartweb.core.testing import ImioSmartwebTestCase
from imio.smartweb.core.tests.utils import clear_cache
from imio.smartweb.core.tests.utils import get_json
from imio.smartweb.core.tests.utils import make_named_image
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.textfield.value import RichTextValue
from plone.base.utils import safe_text
from plone.memoize.ram import choose_cache
from plone.namedfile.file import NamedBlobImage
from unittest.mock import patch
from unittest.mock import Mock
from zope.annotation import IAnnotations
from zope.component import getMultiAdapter

import requests_mock


class TestPage(ImioSmartwebTestCase):
    layer = IMIO_SMARTWEB_CORE_FUNCTIONAL_TESTING

    @freeze_time("2024-02-02 8:00:00")
    def setUp(self):
        self.request = self.layer["request"]
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        api.portal.set_registry_record("plone.enable_sitemap", True)

        self.default_page = api.content.create(
            container=self.portal,
            type="imio.smartweb.PortalPage",
            title="Portal page",
            id="portal-page",
        )
        self.portal.setDefaultPage("portal-page")

        self.folder = api.content.create(
            container=self.portal,
            type="imio.smartweb.Folder",
            title="Folder",
            id="folder",
        )
        self.page = api.content.create(
            container=self.folder,
            type="imio.smartweb.Page",
            title="Page 1",
            id="page1",
        )
        self.section_text = api.content.create(
            container=self.page,
            type="imio.smartweb.SectionText",
            title="Section text",
        )
        self.section_text.text = RichTextValue(
            "<p>Kamoulox</p>", "text/html", "text/html"
        )

        self.rest_directory = api.content.create(
            container=self.portal,
            type="imio.smartweb.DirectoryView",
            title="directory view",
        )
        self.rest_agenda = api.content.create(
            container=self.portal,
            type="imio.smartweb.EventsView",
            title="agenda view",
        )
        self.rest_agenda.selected_agenda = "64f4cbee9a394a018a951f6d94452914"
        self.rest_news = api.content.create(
            container=self.portal,
            type="imio.smartweb.NewsView",
            title="news view",
        )
        self.rest_news.selected_news_folder = "64f4cbee9a394a018a951f6d94452914"
        self.json_rest_directory = get_json("resources/json_rest_directory.json")
        self.json_rest_events = get_json("resources/json_rest_events.json")
        self.json_rest_news = get_json("resources/json_rest_news.json")

        api.content.transition(self.rest_directory, "publish")
        api.content.transition(self.rest_agenda, "publish")
        api.content.transition(self.rest_news, "publish")

    # 'http://localhost:8080/Plone/@search?selected_entities=396907b3b1b04a97896b12cc792c77f8&portal_type=imio.directory.Contact&fullobjects=0&sort_on=sortable_title'
    @freeze_time("2024-02-02 10:00:00")
    @requests_mock.Mocker()
    def test_sitemap(self, m):
        sitemap = getMultiAdapter(
            (self.portal, self.portal.REQUEST), name="sitemap.xml.gz"
        )
        xml = self.uncompress(sitemap())
        self.assertIn("<lastmod >2024-02-02T08:00:00", xml)
        self.assertIn("<loc>http://nohost/plone/folder</loc>", xml)
        self.assertIn("http://nohost/plone/folder/page1", xml)
        self.assertNotIn(
            "<loc>http://nohost/plone/folder/page1/gallery/image/view</loc>", xml
        )

        # Gallery and image created 2024-02-02 10:00:00
        gallery = api.content.create(
            container=self.page,
            type="imio.smartweb.SectionGallery",
            title="Gallery",
        )
        image = api.content.create(
            container=gallery,
            type="Image",
            title="Image",
        )
        image.image = NamedBlobImage(**make_named_image())
        sitemap = getMultiAdapter(
            (self.portal, self.portal.REQUEST), name="sitemap.xml.gz"
        )
        xml = self.uncompress(sitemap())
        self.assertIn("<loc>http://nohost/plone/folder/page1/gallery/image</loc>", xml)
        # Gallery and image created 2024-02-02 10:00:00
        self.assertIn(
            "<loc>http://nohost/plone/folder/page1/gallery/image</loc>\n    <lastmod >2024-02-02T10:00:00",
            xml,
        )

        cache = choose_cache("imio.smartweb.core.browser.sitemap.get_endpoint_data")
        cache.ramcache.invalidateAll()

        with patch(
            "imio.smartweb.core.contents.rest.news.endpoint.BaseNewsEndpoint.__call__",
            return_value=self.json_rest_news,
        ) as mypatch:
            xml = self.uncompress(sitemap())
            self.assertIn(
                "<loc>http://nohost/plone/news-view/ceci-est-une-deuxieme-actualite",
                xml,
            )

        cache = choose_cache("imio.smartweb.core.browser.sitemap.get_endpoint_data")
        cache.ramcache.invalidateAll()

        with patch(
            "imio.smartweb.core.contents.rest.directory.endpoint.BaseDirectoryEndpoint.__call__",
            return_value=self.json_rest_directory,
        ) as mypatch:
            xml = self.uncompress(sitemap())
            self.assertIn(
                "<loc>http://nohost/plone/directory-view/service-communication-de-ladministration-communale",
                xml,
            )

        cache = choose_cache("imio.smartweb.core.browser.sitemap.get_endpoint_data")
        cache.ramcache.invalidateAll()

        with patch(
            "imio.smartweb.core.contents.rest.events.endpoint.BaseEventsEndpoint.__call__",
            return_value=self.json_rest_events,
        ):
            xml = self.uncompress(sitemap())
            self.assertIn(
                "<loc>http://nohost/plone/agenda-view/evenement-recurrent-tous-les-samedi",
                xml,
            )

    def test_site_map_for_user_display(self):
        sitemap = CatalogSiteMap(self.portal, self.request)
        # 3 authentic sources
        self.assertEqual(len(sitemap.siteMap().get("children")), 3)
        self.assertNotIn(
            "Folder",
            [child.get("Title") for child in sitemap.siteMap().get("children")],
        )

        # Publish folder and page (private content don't appear in sitemap)
        api.content.transition(self.folder, "publish")
        api.content.transition(self.page, "publish")
        sitemap = CatalogSiteMap(self.portal, self.request)
        self.assertEqual(len(sitemap.siteMap().get("children")), 4)
        self.assertIn(
            "Folder",
            [child.get("Title") for child in sitemap.siteMap().get("children")],
        )
        folder_entry = [
            child
            for child in sitemap.siteMap().get("children")
            if child.get("Title") == "Folder"
        ][0]
        self.assertIn(
            "Page 1", [child.get("Title") for child in folder_entry.get("children")]
        )

        directory_entry = [
            child
            for child in sitemap.siteMap().get("children")
            if child.get("Title") == "directory view"
        ][0]
        self.assertEqual(len(directory_entry.get("children")), 0)

        cache = choose_cache("imio.smartweb.core.browser.sitemap.get_endpoint_data")
        cache.ramcache.invalidateAll()

        # Populate directory view with 6 contacts
        with patch(
            "imio.smartweb.core.contents.rest.directory.endpoint.BaseDirectoryEndpoint.__call__",
            return_value=self.json_rest_directory,
        ):
            sitemap = CatalogSiteMap(self.portal, self.request)
            directory_entry = [
                child
                for child in sitemap.siteMap().get("children")
                if child.get("Title") == "directory view"
            ][0]
            self.assertEqual(len(directory_entry.get("children")), 6)

    def test_bad_portal_type(self):
        obj = Mock()
        obj.portal_type = None
        request = Mock()
        result = get_endpoint_data(obj, request)
        assert result == {}

    def uncompress(self, sitemapdata):
        sio = BytesIO(sitemapdata)
        unzipped = GzipFile(fileobj=sio)
        xml = unzipped.read()
        unzipped.close()
        return safe_text(xml)



# -*- coding: utf-8 -*-

from imio.smartweb.locales import SmartwebMessageFactory as _
from plone.autoform import directives
from plone.autoform.interfaces import IFormFieldProvider
from plone.supermodel import model
from zope import schema
from zope.interface import provider


@provider(IFormFieldProvider)
class INewTab(model.Schema):

    directives.order_after(open_in_new_tab="remoteUrl")
    open_in_new_tab = schema.Bool(
        title=_("Open in a new tab"),
        required=False,
        default=False,
    )



# -*- coding: utf-8 -*-

from imio.smartweb.core.contents.sections.views import CarouselOrTableSectionView
from imio.smartweb.core.contents.sections.views import HashableJsonSectionView
from imio.smartweb.core.utils import batch_results
from imio.smartweb.core.utils import get_iadeliberation_institution_from_registry
from imio.smartweb.core.utils import get_iadeliberation_json
from imio.smartweb.core.utils import hash_md5
from imio.smartweb.core.utils import remove_cache_key


class TimestampedPublicationsView(CarouselOrTableSectionView, HashableJsonSectionView):
    """TimestampedPublications Section view"""

    @property
    def items(self):
        iadeliberation_institution = get_iadeliberation_institution_from_registry()
        max_items = self.context.nb_results_by_batch * self.context.max_nb_batches
        selected_item = "&UID=".join(self.context.related_timestamped_publications)
        modified_hash = hash_md5(str(self.context.modification_date))
        params = [
            selected_item,
            "portal_type=Publication",
            "fullobjects=y",
            "metadata_fields=UID",
            "metadata_fields=id",
            "review_state=published",
            "sort_on=sortable_title",
            "metadata_fields=modified",
            "metadata_fields=effective",
            f"cache_key={modified_hash}",
            f"sort_limit={max_items}",
        ]
        url = "{}/@search?UID={}".format(iadeliberation_institution, "&".join(params))
        self.json_data = get_iadeliberation_json(url)
        self.json_data = remove_cache_key(self.json_data)
        self.refresh_modification_date()
        if self.json_data is None or len(self.json_data.get("items", [])) == 0:
            return []
        items = self.json_data.get("items")[:max_items]
        results = []
        for item in items:
            type_document = None
            category = None
            if item.get("document_type"):
                type_document = item.get("document_type").get("title")
            if item.get("category"):
                category = item.get("category").get("title")
            dict_item = {
                "uid": item.get("UID"),
                "title": item.get("title"),
                "description": item.get("description"),
                "url": item.get("@id"),
                "effective": item.get("effective"),
                "publication_document_type": type_document,
                "publication_category": category,
                "publication_attached_file": item.get("file"),
                "has_image": False,
            }
            results.append(dict_item)
        return batch_results(results, self.context.nb_results_by_batch)

    @property
    def see_all_url(self):
        return self.context.linking_rest_view.to_object.absolute_url()



# -*- coding: utf-8 -*-
from imio.smartweb.core.contents.rest.campaign.endpoint import CampaignEndpoint
from imio.smartweb.core.contents.rest.campaign.endpoint import (
    PROJECT_WORKFLOW_STATUS_TO_KEEP,
)
from imio.smartweb.core.contents import ICampaignView
from imio.smartweb.core.tests.utils import get_json

from imio.smartweb.core.testing import IMIO_SMARTWEB_CORE_INTEGRATION_TESTING
from imio.smartweb.core.testing import ImioSmartwebTestCase
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.dexterity.interfaces import IDexterityFTI
from unittest.mock import patch
from zope.component import createObject
from zope.component import queryUtility
from zope.publisher.browser import TestRequest

import json
import requests_mock


class TestIdeabox(ImioSmartwebTestCase):
    layer = IMIO_SMARTWEB_CORE_INTEGRATION_TESTING

    def setUp(self):
        """Custom shared utility setup for tests."""
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.folder = api.content.create(
            container=self.portal,
            type="imio.smartweb.Folder",
            title="A Folder",
        )
        self.json_campaigns_raw_mock = get_json("resources/json_ideabox_campaigns.json")
        self.json_campaign_raw_mock = get_json("resources/json_ideabox_campaign.json")

    def test_ct_publication_schema(self):
        fti = queryUtility(IDexterityFTI, name="imio.smartweb.CampaignView")
        schema = fti.lookupSchema()
        self.assertEqual(ICampaignView, schema)

    def test_ct_publication_fti(self):
        fti = queryUtility(IDexterityFTI, name="imio.smartweb.CampaignView")
        self.assertTrue(fti)

    def test_ct_publication_factory(self):
        fti = queryUtility(IDexterityFTI, name="imio.smartweb.CampaignView")
        factory = fti.factory
        obj = createObject(factory)
        self.assertTrue(
            ICampaignView.providedBy(obj),
            "ICampaignView not provided by {0}!".format(
                obj,
            ),
        )

    @patch("imio.smartweb.core.subscribers.get_basic_auth_json")
    @patch("imio.smartweb.core.subscribers.get_value_from_registry")
    @patch("imio.smartweb.core.contents.rest.campaign.content.get_basic_auth_json")
    @patch("imio.smartweb.core.contents.rest.campaign.content.get_value_from_registry")
    def test_get_campaign(
        self,
        m_get_value_from_registry_namechooser,
        m_get_basic_auth_json_namechooser,
        m_get_value_from_registry,
        m_get_basic_auth_json,
    ):
        m_get_value_from_registry.return_value = (
            "https://staging3-formulaires.guichet-citoyen.be/api"
        )
        m_get_basic_auth_json.return_value = self.json_campaign_raw_mock

        m_get_value_from_registry_namechooser.return_value = (
            "https://staging3-formulaires.guichet-citoyen.be/api"
        )
        m_get_basic_auth_json_namechooser.return_value = self.json_campaign_raw_mock
        campaign_view = api.content.create(
            title="kamoulox",
            container=self.folder,
            type="imio.smartweb.CampaignView",
            linked_campaign="2",
        )
        self.assertEqual(campaign_view.title, "Sprint iMio Fall 2024")
        self.assertEqual(campaign_view.id, "sprint-imio-fall-2024")

    # CampaignNameChooser : Give th Title to our object thanks to e-guichet object Title.
    @patch("imio.smartweb.core.subscribers.get_basic_auth_json")
    @patch("imio.smartweb.core.subscribers.get_value_from_registry")
    @patch("imio.smartweb.core.contents.rest.campaign.content.get_basic_auth_json")
    @patch("imio.smartweb.core.contents.rest.campaign.content.get_value_from_registry")
    def test_campaign_name_chooser(
        self,
        m_get_value_from_registry_namechooser,
        m_get_basic_auth_json_namechooser,
        m_get_value_from_registry,
        m_get_basic_auth_json,
    ):
        from imio.smartweb.core.contents.rest.campaign.content import CampaignView
        from zope.component import queryAdapter
        from zope.container.interfaces import INameChooser

        m_get_value_from_registry.return_value = (
            "https://staging3-formulaires.guichet-citoyen.be/api"
        )
        m_get_basic_auth_json.return_value = self.json_campaign_raw_mock

        m_get_value_from_registry_namechooser.return_value = (
            "https://staging3-formulaires.guichet-citoyen.be/api"
        )
        m_get_basic_auth_json_namechooser.return_value = self.json_campaign_raw_mock
        campaign_view = CampaignView(id="test-campaign", title=None)
        campaign_view.linked_campaign = "2"
        name_chooser = queryAdapter(self.folder, INameChooser)
        generated_id = name_chooser.chooseName(None, campaign_view)
        self.assertEqual(campaign_view.title, "Sprint iMio Fall 2024")
        self.assertEqual(generated_id, "sprint-imio-fall-2024")

    @requests_mock.Mocker()
    @patch("imio.smartweb.core.subscribers.get_basic_auth_json")
    @patch("imio.smartweb.core.subscribers.get_value_from_registry")
    def test_get_projects(self, m_get_value_from_registry, m_get_basic_auth_json, m):
        m_get_value_from_registry.return_value = (
            "https://staging3-formulaires.guichet-citoyen.be/api"
        )
        m_get_basic_auth_json.return_value = self.json_campaign_raw_mock
        campaign_view = api.content.create(
            id="kamoulox",
            container=self.folder,
            type="imio.smartweb.CampaignView",
            linked_campaign="2",
        )
        # react additionnal fields request.
        # form={
        #         "taxonomy_contact_category_for_filtering": ("token"),
        #         "topics": "education",
        #     }

        request = TestRequest()
        endpoint = CampaignEndpoint(campaign_view, request)
        url = endpoint.query_url
        filter_statut = "|".join(PROJECT_WORKFLOW_STATUS_TO_KEEP)
        self.assertEqual(
            url,
            f"https://demo-formulaires.guichet-citoyen.be/api/cards/imio-ideabox-projet/list?filter-campagne=2&full=on&filter-statut={filter_statut}&filter-statut-operator=in&",
        )

        m.get(url, text=json.dumps({}))
        call = endpoint()
        self.assertEqual(call, {})

        # to be continued



# -*- coding: utf-8 -*-

from imio.smartweb.core.contents.rest.view import BaseRestView
from imio.smartweb.core.interfaces import IOgpViewUtils
from plone import api
from zope.interface import implementer


@implementer(IOgpViewUtils)
class CampaignViewView(BaseRestView):
    """CampaignView view"""

    @property
    def propose_url(self):
        return self.context.propose_project_url

    @property
    def display_map(self):
        return self.context.display_map

    @property
    def display_red_thumbs(self):
        return self.context.display_red_thumbs

    @property
    def local_query_zones_url(self):
        base_url = self.context.absolute_url()
        return f"{base_url}/@zones"

    @property
    def local_query_topics_url(self):
        base_url = self.context.absolute_url()
        return f"{base_url}/@all_topics"



# -*- coding: utf-8 -*-

from imio.smartweb.core.browser.controlpanel import ISmartwebControlPanel
from imio.smartweb.core.contents import IPages
from imio.smartweb.locales import SmartwebMessageFactory as _
from eea.facetednavigation.interfaces import ICriteria
from eea.facetednavigation.subtypes.interfaces import IFacetedNavigable
from plone import api
from plone.app.imagecropping import PAI_STORAGE_KEY
from plone.registry import field
from plone.registry import Record
from plone.registry.interfaces import IRegistry
from zope.annotation.interfaces import IAnnotations
from zope.component import getUtility
from zope.schema import getFieldNames
from zope.schema import TextLine

import logging

logger = logging.getLogger("imio.smartweb.core")
PROFILEID = "profile-imio.smartweb.core:default"


def configure_first_official_release(context):
    portal_setup = api.portal.get_tool("portal_setup")
    portal_setup.runImportStepFromProfile(PROFILEID, "plone.app.registry")
    portal_setup.runImportStepFromProfile(PROFILEID, "typeinfo")

    portal_setup.runImportStepFromProfile(PROFILEID, "catalog")
    portal_catalog = api.portal.get_tool("portal_catalog")
    # we could index only imio.smartweb.Page contents, but for some reason
    # api.content.find() doesn't find all objects
    portal_catalog.clearFindAndRebuild()
    logger.info("Reindexed catalog for category_and_topics index / metadata")


def set_thumb_scales(context):
    api.portal.set_registry_record("plone.thumb_scale_listing", "liste")
    api.portal.set_registry_record("plone.thumb_scale_summary", "vignette")


def update_actions(context):
    portal_setup = api.portal.get_tool("portal_setup")
    portal_setup.runImportStepFromProfile(PROFILEID, "actions")
    portal_actions = api.portal.get_tool("portal_actions")
    header_actions = getattr(portal_actions, "header_actions")
    if "e_guichet" in header_actions.objectIds():
        header_actions._delObject("e_guichet")
        logger.info("Deleted e_guichet header action")


def update_icons_and_names(context):
    portal_setup = api.portal.get_tool("portal_setup")
    portal_setup.runImportStepFromProfile(
        "profile-imio.smartweb.core:icons-basic", "plone.app.registry"
    )
    registry = getUtility(IRegistry)
    for old_icon in [
        "annuaire",
        "demarches",
        "ecoles",
        "horaires",
        "mobilite",
        "parkings",
        "sports",
        "tourisme",
        "travaux",
    ]:
        if f"smartweb.icon.{old_icon}" in registry:
            del registry.records[f"smartweb.icon.{old_icon}"]


def reload_registry(context):
    portal_setup = api.portal.get_tool("portal_setup")
    portal_setup.runImportStepFromProfile(PROFILEID, "plone.app.registry")


def reload_rolemap(context):
    portal_setup = api.portal.get_tool("portal_setup")
    portal_setup.runImportStepFromProfile(PROFILEID, "rolemap")


def reload_types(context):
    portal_setup = api.portal.get_tool("portal_setup")
    portal_setup.runImportStepFromProfile(PROFILEID, "typeinfo")


def reload_workflows(context):
    portal_setup = api.portal.get_tool("portal_setup")
    portal_setup.runImportStepFromProfile(PROFILEID, "workflow")


def check_itinerary_if_address_is_checked(context):
    brains = api.content.find(portal_type="imio.smartweb.SectionContact")
    for brain in brains:
        obj = brain.getObject()
        if "address" in obj.visible_blocks:
            if "itinerary" not in obj.visible_blocks:
                obj.visible_blocks.append("itinerary")
                obj._p_changed = 1


def add_placeholder_to_faceted_textsearch(context):
    brains = api.content.find(object_provides=IFacetedNavigable)
    for brain in brains:
        obj = brain.getObject()
        handler = ICriteria(obj)
        for criterion in handler.criteria:
            if criterion.widget == "text":
                criterion.placeholder = "Recherche"
                handler.criteria._p_changed = 1


def exclude_footers_from_parent_listing(context):
    brains = api.content.find(portal_type="imio.smartweb.Footer")
    for brain in brains:
        footer = brain.getObject()
        footer.exclude_from_parent_listing = True
        footer.reindexObject(idxs=["exclude_from_parent_listing"])


def reindex_all_pages(context):
    brains = api.content.find(object_provides=IPages)
    for brain in brains:
        try:
            obj = brain.getObject()
        except KeyError:
            logger.warn(f"getObject failed on {brain.getURL()}")
        else:
            obj.reindexObject()


def add_sendinblue_button_settings(context):
    fields = getFieldNames(ISmartwebControlPanel)
    fields.remove("sendinblue_button_position")
    fields.remove("sendinblue_button_text")
    registry = getUtility(IRegistry)
    registry.registerInterface(ISmartwebControlPanel, omit=fields, prefix="smartweb")


def find_multiple_categories_directory_views(context):
    brains = api.content.find(portal_type="imio.smartweb.DirectoryView")
    for brain in brains:
        directory = brain.getObject()
        if (
            directory.selected_categories is None
            or len(directory.selected_categories) <= 1
        ):
            continue
        msg = f"Found directory view with multiple categories : {directory.absolute_url()}"
        logger.warning(msg)
        api.portal.send_email(
            recipient="boulch@imio.be",
            subject="Multiple contacts categories in directory view",
            body=msg,
        )


def to_related_contacts(context):
    brains = api.content.find(portal_type="imio.smartweb.SectionContact")
    for brain in brains:
        obj = brain.getObject()
        if hasattr(obj, "related_contact"):
            setattr(obj, "related_contacts", [obj.related_contact])
            setattr(obj, "nb_contact_by_line", 1)
            delattr(obj, "related_contact")


def remove_deprecated_cropping_annotations(context):
    with api.env.adopt_user(username="admin"):
        brains = api.content.find(portal_type="imio.smartweb.Folder")
        for brain in brains:
            obj = brain.getObject()
            annotations = IAnnotations(obj)
            scales = annotations.get(PAI_STORAGE_KEY)
            if scales is not None and "banner_banner" in scales:
                del scales["banner_banner"]
                obj.reindexObject()
                logger.info(
                    f"Remove deprecated banner_banner cropping annotation on {obj.absolute_url()}"
                )


def migrate_is_in_portrait_mode(context):
    with api.env.adopt_user(username="admin"):
        brains = api.content.find(portal_type="imio.smartweb.SectionContact")
        for brain in brains:
            obj = brain.getObject()
            is_in_portrait_mode = getattr(obj, "is_in_portrait_mode", False)
            if is_in_portrait_mode:
                obj.orientation = "portrait"
                logger.info(
                    f"Migrated potrait mode to orientation portrait for {obj.absolute_url()}"
                )


def migrate_old_scales_from_vocabulary(context):
    with api.env.adopt_user(username="admin"):
        brains = api.content.find(
            portal_type=[
                "imio.smartweb.SectionContact",
                "imio.smartweb.SectionGallery",
            ]
        )
        for brain in brains:
            obj = brain.getObject()
            old_scale = obj.image_scale
            if old_scale in ["affiche", "vignette", "liste"]:
                continue
            new_scale = "affiche"
            if old_scale == "preview":
                new_scale = "vignette"
            obj.image_scale = new_scale
            logger.info(
                f"Migrated deprecated scale from {old_scale} to {new_scale} for {obj.absolute_url()}"
            )


def migrate_old_sizes_from_section_text(context):
    with api.env.adopt_user(username="admin"):
        brains = api.content.find(portal_type=["imio.smartweb.SectionText"])
        for brain in brains:
            obj = brain.getObject()
            old_scale = obj.image_size
            if old_scale in ["affiche", "vignette"]:
                continue
            new_scale = "affiche"
            if old_scale == "preview":
                new_scale = "vignette"
            obj.image_size = new_scale
            logger.info(
                f"Migrated deprecated scale from {old_scale} to {new_scale} for {obj.absolute_url()}"
            )


def update_control_panel_combo_api_url(context):
    url_ts = ""
    try:
        url_ts = api.portal.get_registry_record("smartweb.url_formdefs_api") or ""
    except:
        logger.info("La clé 'smartweb.url_formdefs_api' a été supprimée.")
        return
    url_ts = url_ts.replace("/api", "")
    url_ts = url_ts.replace("-formulaires", "")
    api.portal.set_registry_record("smartweb.url_formdefs_api", url_ts)


def update_control_panel_combo_api_fieldname(context):
    url_ts = ""
    try:
        url_ts = api.portal.get_registry_record("smartweb.url_formdefs_api") or ""
    except:
        logger.info("La clé 'smartweb.url_formdefs_api' a été supprimée.")
    registry = getUtility(IRegistry)
    records = registry.records
    if "smartweb.url_ts" in records:
        return
    logger.info("Adding smartweb.url_ts to registry")  # noqa
    record = Record(
        field.TextLine(
            title=_("Url to e-guichet"),
            description=_("Exemple : https://COMMUNE.guichet-citoyen.be"),
            required=False,
        ),
        value=url_ts,
    )
    records["smartweb.url_ts"] = record

    record = Record(
        field.TextLine(
            title=_(
                "Username to consume e-guichet ideabox API (get Campaign, projects,...)"
            ),
            default="ideabox",
            required=False,
        )
    )
    records["smartweb.iaideabox_api_username"] = record

    record = Record(
        field.Password(
            title=_(
                "Password to consume e-guichet ideabox API (get Campaign, projects,...)"
            ),
            required=False,
        )
    )
    records["smartweb.iaideabox_api_password"] = record
    try:
        del registry.records["smartweb.url_formdefs_api"]
        logger.info("La clé 'smartweb.url_formdefs_api' a été supprimée.")
    except KeyError:
        logger.info("La clé 'smartweb.url_formdefs_api' n'existe pas dans le registre.")



# -*- coding: utf-8 -*-

from imio.smartweb.common.widgets.select import TranslatedAjaxSelectWidget
from imio.smartweb.core.contents.sections.base import ISection
from imio.smartweb.core.contents.sections.base import Section
from imio.smartweb.locales import SmartwebMessageFactory as _
from plone.autoform import directives
from plone.supermodel import model
from zope import schema
from zope.interface import implementer


class ISectionTimestampedPublications(ISection):
    """Marker interface and Dexterity Python Schema for SectionTimestampedPublications"""

    related_timestamped_publications = schema.List(
        title=_("Related timestamped publications"),
        value_type=schema.Choice(
            source="imio.smartweb.vocabulary.IADeliberationsPublications"
        ),
        required=False,
    )
    directives.widget(
        "related_timestamped_publications",
        TranslatedAjaxSelectWidget,
        vocabulary="imio.smartweb.vocabulary.IADeliberationsPublications",
        pattern_options={"multiple": True},
    )

    nb_results_by_batch = schema.Choice(
        title=_("Number of items per batch"),
        required=True,
        default=3,
        values=[1, 2, 3, 4],
    )

    max_nb_batches = schema.Int(
        title=_("Maximum number of batches to display"),
        required=True,
        default=2,
        min=1,
        max=12,
    )

    model.fieldset("layout", fields=["show_items_description"])
    show_items_description = schema.Bool(
        title=_("Show items description"), required=False
    )


@implementer(ISectionTimestampedPublications)
class SectionTimestampedPublications(Section):
    """SectionTimestampedPublications class"""

    # We don't need to manage display on timestamped publications
    # only common table view / don't need common carousel view
    manage_display = False
    show_items_date = True



# -*- coding: utf-8 -*-

from imio.smartweb.core.config import DIRECTORY_URL
from imio.smartweb.core.contents.sections.contact.utils import ContactProperties
from imio.smartweb.core.contents.sections.views import HashableJsonSectionView
from imio.smartweb.core.utils import batch_results
from plone import api
from zope.component import queryMultiAdapter


class ContactView(HashableJsonSectionView):
    """Contact Section view"""

    def get_number_of_contacts(self):
        """
        Returns the number of related contacts.
        """
        related_contacts = (
            self.context.related_contacts
        )  # Assurez-vous que 'related_contacts' est une liste
        if related_contacts:
            return len(related_contacts)
        return 0

    def contacts(self):
        # Firstly, try to get contact from the container view
        container_view = queryMultiAdapter(
            (self.context.aq_parent, self.request), name="full_view"
        )
        self.json_data = container_view.get_page_contacts()
        if self.json_data is None or len(self.json_data.get("items", [])) == 0:
            return []

        container_contacts = self.json_data.get("items")
        results_items = [
            contact
            for contact in container_contacts
            if contact["UID"] in self.context.related_contacts
        ]
        index_map = {
            value: index for index, value in enumerate(self.context.related_contacts)
        }
        results_items = sorted(results_items, key=lambda x: index_map[x["UID"]])

        # construct JSON data as before WEBBDC-1265 to avoid hash differences
        uids = "&UID=".join(self.context.related_contacts)
        url = "{}/@search?UID={}&fullobjects=1".format(DIRECTORY_URL, uids)
        current_lang = api.portal.get_current_language()[:2]
        if current_lang != "fr":
            url = f"{url}&translated_in_{current_lang}=1"
        self.json_data = {
            "@id": url,
            "items": results_items,
            "items_total": len(results_items),
        }
        self.refresh_modification_date()
        return batch_results(results_items, self.context.nb_contact_by_line)

    def get_contact_properties(self, json_dict):
        return ContactProperties(json_dict, self.context)



# -*- coding: utf-8 -*-

from eea.facetednavigation.browser.app.view import FacetedContainerView
from eea.facetednavigation.layout.interfaces import IFacetedLayout
from imio.smartweb.core.interfaces import IViewWithoutLeadImage
from imio.smartweb.core.utils import get_scale_url
from imio.smartweb.locales import SmartwebMessageFactory as _
from plone import api
from plone.app.contenttypes.browser.folder import FolderView
from plone.app.contenttypes.interfaces import IFile
from zope.component import queryAdapter
from zope.i18n import translate
from zope.interface import implementer


@implementer(IViewWithoutLeadImage)
class SmartwebFacetedContainerView(FacetedContainerView):
    """Faceted view without lead image"""


class FacetedView(FolderView):
    """Faceted common view"""

    @property
    def layout(self):
        context = self.context
        current_layout = queryAdapter(context, IFacetedLayout).layout
        return current_layout

    def show_images(self):
        return "with-images" in self.layout

    def is_video(self, item):
        return item.portal_type == "imio.smartweb.SectionVideo"

    def is_target_blank(self, item):
        # don't wake up object
        if item.portal_type == "File":
            return True
        # if IFile.providedBy(item.getObject()):
        #     return True
        return False

    def target(self, item):
        if self.is_target_blank(item):
            return "_blank"
        return ""

    def a_tag_item_title(self, item):
        title = item.Title or ""
        if self.is_target_blank(item):
            current_lang = api.portal.get_current_language()[:2]
            new_tab_txt = translate(_("New tab"), target_language=current_lang)
            return f"{title} ({new_tab_txt})"
        return title

    def get_scale_url(self, item, scale="vignette"):
        orientation = self.context.orientation
        if item.portal_type == "imio.smartweb.SectionGallery":
            images = item.getObject().listFolderContents()
            if not images:
                return ""
            scale_url = get_scale_url(
                images[0], self.request, "image", scale, orientation
            )
            return scale_url
        return get_scale_url(item, self.request, "image", scale, orientation)
